//---------------------------------------------------------------------
//	File:		4dgtLCD.c
//	Status:		Beta
//
// Written By:		Nicola Ferrarese						
// MPLAB IDE 7.50.00.00
//
// The code was compiled using the HI-TECH PICC compiler v. 8.02PL1 Demo.
//
// Project > Buil Options.. > Project,
// "PICC Compiler" Tab, check "Enable assembler optimization" option.
//
//		
//
//---------------------------------------------------------------------
//	HEADER FILE INCLUDES
//---------------------------------------------------------------------
#include "delay.h"
#include <pic1687x.h>			// Register definitions
#include "com_types.h"			// Common type definitions

//---------------------------------------------------------------------
//	Port Pin Designations
//---------------------------------------------------------------------
static bit CLK       @ BITNUM(PORTA, 0);	// CLOCK 	for LCD Driver
static bit DATAIN    @ BITNUM(PORTA, 1);	// DATAIN 	for LCD Driver
static bit LOAD      @ BITNUM(PORTA, 2);	// LOAD Data Line  for LCD Driver

//---------------------------------------------------------------------
//	Variable declarations
//---------------------------------------------------------------------
static union chartype
dgt1,dgt2,dgt3,dgt4,
seg1,seg2,seg3,seg4,
dsr1,dsr2,dsr3,dsr4;

//---------------------------------------------------------------------
// Function Prototypes Declaration
//---------------------------------------------------------------------
extern void updatelcd(unsigned int, char dsplyz);	// refresh display 
static void bin2BCD16(unsigned int);				// convert a 16bit integer to BCD format
static unsigned char decode(unsigned char);			// make bit pattern from a digit value						
static void perm32(void);							// permutate segment to match PCB layout
static void send8(unsigned char);					// send a digit (8 segments)
static void killzero(void);							// remove no significant zero Ex. 0001 -> 1

//---------------------------------------------------------------------
// Update LCD
//---------------------------------------------------------------------
extern void updatelcd(unsigned int num, char nozero)
{
bin2BCD16 (num);
seg1.byte = decode(dgt1.byte);		// decode digit 1
seg2.byte = decode(dgt2.byte);		// decode digit 2
seg3.byte = decode(dgt3.byte);		// decode digit 3
seg4.byte = decode(dgt4.byte);		// decode digit 4
if(nozero) killzero();				// supress no significant zero
perm32();							// permutate 32 bit from segx to dgtx
send8(dsr4.byte);					
send8(dsr3.byte);
send8(dsr2.byte);
send8(dsr1.byte);
}

//---------------------------------------------------------------------
// Send 8 bit serially
//---------------------------------------------------------------------
static void send8(unsigned char c)
{
int i;					
for(i=0; i<8; i++)
	{
	CLK=HIGH;
	if (c&0x01) DATAIN=1;	// send LSb to DATAIN pin
		else DATAIN=0;
	c = c >>1;
	CLK=LOW;
	}
		
LOAD=HIGH;
DelayUs(10);
LOAD=LOW;
}

//-------------------------------------------------------------------------;
//   Questa tabella restituisce il bit pattern per i segmenti, per dato
//          	 un dato valore in w compreso tra 0 e 31       
//-------------------------------------------------------------------------;
static unsigned char decode(unsigned char c)
{
unsigned char decodedgt;
switch(c)
	{
	case(0):
		decodedgt=0b00111111;	// 0
		break;
	case(1):
		decodedgt=0b00000110;	// 1
		break;
	case(2):
		decodedgt=0b01011011;	// 2
		break;
	case(3):				
		decodedgt=0b01001111;	// 3
		break;
	case(4):
		decodedgt=0b01100110;	// 4
		break;
	case(5):
		decodedgt=0b01101101;	// 5
		break;
	case(6):
		decodedgt=0b01111101;	// 6
		break;
	case(7):
		decodedgt=0b00000111;	// 7
		break;
	case(8):
		decodedgt=0b01111111;	// 8
		break;
	case(9):
		decodedgt=0b01101111;	// 9
		break;
	case(10):
		decodedgt=0b01110111;	// A	10
		break;
	case(11):
		decodedgt=0b01111100;	// b	11
		break;
	case(12):
		decodedgt=0b00111001;	// C	12
		break;
	case(13):
		decodedgt=0b01011110;	// d	13
		break;
	case(14):
		decodedgt=0b01111001;	// E	14
		break;
	case(15):
		decodedgt=0b01110001;	// F	15
		break;
	case(16):
		decodedgt=0b00000000;	// blanck 16
		break;
	case(17):
		decodedgt=0b01110110;	// H	17
		break;
	case(18):
		decodedgt=0b00111000;	// L	18
		break;
	case(19):
		decodedgt=0b01110011;	// P	19
		break;
	case(20):
		decodedgt=0b00111110;	// U	20
		break;
	case(21):
		decodedgt=0b00110000;	// |	21
		break;
	case(22):
		decodedgt=0b01000000;	//	-	22
		break;
	case(23):
		decodedgt=0b01100011;	// 	23
		break;
	case(24):
		decodedgt=0b01011100;	// o	24
		break;
	case(25):
		decodedgt=0b11111111;	// 8.	all segment on, with point
		break;
	default:
		decodedgt=0b01111001;	// E
	}
return decodedgt;
}

//-------------------------------------------------------------------------
//	PERMUTE SEGMENT TO MATCH PCB LAYOUT
//-------------------------------------------------------------------------
static void perm32(void)
{
/*
        ---- SEG1 -----  ---- SEG2 -----  ---- SEG3 -----  ---- SEG4 -----
        7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0
 
        1 1 1 1 1 1 1 1  2 2 2 2 2 2 2 2  3 3 3 3 3 3 3 3  4 4 4 4 4 4 4 4 
        P G F E D C B A  P G F E D C B A  P G F E D C B A  : G F E D C B A

        ---- DSR1 -----  ---- DSR2 -----  ---- DSR3 -----  ---- DSR4 -----
        7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0  7 6 5 4 3 2 1 0

PCB     1 1 1 1 2 2 2 2  : 3 3 3 3 4 4 4  4 4 4 4 3 3 3 3  2 2 2 2 1 1 1 1
        G F A B G F A B    G F A B G F A  B C D E P C D E  P C D E P C D E

SEG     1 2 3 4 5 6 7 8  9 1 1 1 1 1 1 1  1 1 1 2 2 2 2 2  2 2 2 2 2 3 3 3
					  	   0 1 2 3 4 5 6  7 8 9 0 1 2 3 4  5 6 7 8 9 0 1 2	
*/
dsr1.part.bit7 = seg1.part.bit6;  	// 1G
dsr1.part.bit6 = seg1.part.bit5;  	// 1F
dsr1.part.bit5 = seg1.part.bit0;  	// 1A
dsr1.part.bit4 = seg1.part.bit1;  	// 1B
dsr1.part.bit3 = seg2.part.bit6;  	// 2G
dsr1.part.bit2 = seg2.part.bit5;  	// 2F
dsr1.part.bit1 = seg2.part.bit0;  	// 2A
dsr1.part.bit0 = seg2.part.bit1;  	// 2B

dsr2.part.bit7 = seg4.part.bit7;	// :
dsr2.part.bit6 = seg3.part.bit6;	// 3G
dsr2.part.bit5 = seg3.part.bit5;	// 3F
dsr2.part.bit4 = seg3.part.bit0;	// 3A
dsr2.part.bit3 = seg3.part.bit1;	// 3B
dsr2.part.bit2 = seg4.part.bit6;	// 4G
dsr2.part.bit1 = seg4.part.bit5;	// 4F
dsr2.part.bit0 = seg4.part.bit0;	// 4A

dsr3.part.bit7 = seg4.part.bit1;	// 4B
dsr3.part.bit6 = seg4.part.bit2;	// 4C
dsr3.part.bit5 = seg4.part.bit3;	// 4D
dsr3.part.bit4 = seg4.part.bit4;	// 4E
dsr3.part.bit3 = seg3.part.bit7;	// 3P
dsr3.part.bit2 = seg3.part.bit2;	// 3C
dsr3.part.bit1 = seg3.part.bit3;	// 3D
dsr3.part.bit0 = seg3.part.bit4;	// 3E

dsr4.part.bit7 = seg2.part.bit7;	// 2P
dsr4.part.bit6 = seg2.part.bit2;	// 2C
dsr4.part.bit5 = seg2.part.bit3;	// 2D
dsr4.part.bit4 = seg2.part.bit4;	// 2E
dsr4.part.bit3 = seg1.part.bit7;	// 1P
dsr4.part.bit2 = seg1.part.bit2;	// 1C
dsr4.part.bit1 = seg1.part.bit3;	// 1D
dsr4.part.bit0 = seg1.part.bit4;	// 1E
}


//---------------------------------------------------------------------
//	EXTRACT 4 DIGIT BCD from 16 bit INTEGER
//---------------------------------------------------------------------
static void bin2BCD16(unsigned int binary)
{
union {
  struct {
    unsigned int low;
    unsigned int high;
  } word;
  char byte[4];
} cdec;

unsigned char c;
unsigned char d;

if (binary>9999)		
	{					
	dgt1.byte=14;		// Over range display EEEE
	dgt2.byte=14;
	dgt3.byte=14;
	dgt4.byte=14;
	return;
	}	

cdec.word.high=0;
cdec.word.low=binary;
 
for (c=0; c<16; c++)
	{
	if ((cdec.byte[2]&0x0F)>=0x05) cdec.byte[2]+=0x03;
	if ((cdec.byte[2]&0xF0)>=0x50) cdec.byte[2]+=0x30;
	if ((cdec.byte[3]&0x0F)>=0x05) cdec.byte[3]+=0x03;
	if ((cdec.byte[3]&0xF0)>=0x50) cdec.byte[3]+=0x30;

	d=cdec.byte[1];							// shift hi:lo
	cdec.word.low<<=1;
	cdec.word.high<<=1;
	if ((d&0x80)==0x80) cdec.byte[2]|=1;
	}
// extract BCD and put to digit display register
dgt1.byte=cdec.byte[3]>>4;
dgt2.byte=cdec.byte[3]&0x0F;
dgt3.byte=cdec.byte[2]>>4;
dgt4.byte=cdec.byte[2]&0x0F;
}


//---------------------------------------------------------------------
//	Remove no significant zeros
//---------------------------------------------------------------------
static void killzero(void)
{
if(dgt1.byte==0)
	{
	seg1.byte&=0x80;
	if(dgt2.byte==0)
		{
		seg2.byte&=0x80;
		if(dgt3.byte==0) seg3.byte&=0x80;
		}	
	}	
}
