//---------------------------------------------------------------------
//	File:		main.c
//	Status:		r00
//
// Written By:		Nicola Ferrarese						
// This code implements a 0...9999 rpm meter
// The code was compiled using the HI-TECH PICC compiler v. 8.02PL1 Demo.
//
//
// Base Time 
// fosc		finstr		T0Pres		Tmr0	base0	base1	base2	
// 4MHz		1MHz		x16			x250	x10		x5		x2				
// 1us		1us			16us		4ms  	40ms	200ms 	400ms
//
//
//---------------------------------------------------------------------
// inserire funzioni tasti
// memorizzare rpm max
// memorizzare in eeprom divisore impulsi
// aggiungere funzione power save
//
//---------------------------------------------------------------------
//	HEADER FILE INCLUDES
//---------------------------------------------------------------------
#include <pic16f6x.h>		// Register definitions
#include <pic.h>			// 
#include "delay.h"
#include "com_types.h"
#include "4dgtLCD.h"

//---------------------------------------------------------------------
//	Macros
//---------------------------------------------------------------------

//		Setup of PIC configuration fuses

__CONFIG(UNPROTECT & LVPDIS & BORDIS & MCLREN &
		 PWRTEN & WDTDIS & INTIO );

__EEPROM_DATA(0, 1, 2, 3, 4, 5, 6, 7);								


//---------------------------------------------------------------------
//	Port Pin Designations
//---------------------------------------------------------------------
static bit S0	     @ BITNUM(PORTB, 0);
static bit S1	     @ BITNUM(PORTB, 1);
static bit S2	     @ BITNUM(PORTB, 2);
static bit TEST	     @ BITNUM(PORTB, 4);


//---------------------------------------------------------------------
//	Constant Definitions
//---------------------------------------------------------------------
#define	TMAX	128			// 128*4ms => Tmax=512ms => rpm_min~=120rpm 


//---------------------------------------------------------------------
//	Variable declarations
//---------------------------------------------------------------------

static unsigned char 
intcnt0,intcnt1,intcnt2,				// Interrupt counters
tmax_cnt;								// TMAX counter

static unsigned int
num,									// memorizza il numero da rappresentare sul display LCD a 4 cifre
capture,
capture_old,
capture_new;	

static struct eight_bits basetime;


//---------------------------------------------------------------------
// Function Prototypes Dichiarazione
//---------------------------------------------------------------------
void main(void);				//
void SetUp(void);				// Initialize ports/peripherals 
void InitVar(void);				// Initialize variable
void interrupt Isr(void);		// Interrupt Service Routine
void basetime0(void);
void basetime1(void);
void basetime2(void);
void getkeys(void); 
void calcrpm(void);


//---------------------------------------------------------------------
// Main program
//---------------------------------------------------------------------
void main()
{
SetUp();						// Initialize PORT and Peripheal
InitVar();						// Initialize variable

while(1)						// This is the main program loop 
	{	
	if (basetime.bit0) basetime0();
	if (basetime.bit1) basetime1();
	if (basetime.bit2) basetime2();

	if(CCP1IF)										// polling CCP interrupt flag
		{
		CCP1IF=0;									// clear interrupt flag
		capture_new=CCPR1H;							// load captured value
		capture_new<<=8;
		capture_new+=CCPR1L;
		if(tmax_cnt)
			{
			capture=capture_new-capture_old;		// calculate how many TCK1 periods 
			capture_old=capture_new;				// save value for next calculation
			}
		else capture_old=capture_new;				// save captured value for next calculation
		tmax_cnt=TMAX;								// preset max period counter			
		}											// end if(CCP1IF)

	}							// end while(1)
}								// end main										

	
//---------------------------------------------------------------------
//	Setup() initializes peripheral registers
//---------------------------------------------------------------------

void SetUp(void)		// 0 output, 1 input
{
TRISA = 0x00;			// PORTA input output configuration
PORTA = 0x00;			// Clear PORTA -  All pins Low

TRISB = 0x0F;			// PORTB input output configuration
PORTB = 0;				// Clear PORTB

TMR0=0;					// Clear timers

CM2=1;					// Disable comparator modules
CM1=1;
CM0=1;

TMR1ON=1;				// Enable timer 1
TMR1CS=0;				// Clock source for timer 1 is internal Fosc/4
T1CKPS1=1;				// timer 1 prescaler 
T1CKPS0=1;				// ratio is 1:8

CCP1M3=0;				// Enable capture 
CCP1M2=1;				// mode on CCP1,
CCP1M1=0;				// Capture every 
CCP1M0=0;				// rising edge

					// OPTION REGISTER	
T0CS=0;				// TMRO internal CLK source
PSA=0;				// Prescaler assigned to TMR0 module
PS2=0;				//
PS1=1;				// Prescaler rate 1:16
PS0=1;				//

RBPU=0;				// Enable PORTB pull-ups internal resistor

INTCON=0;			// Disable all interrupts		
PIE1=0x00;			// Disable all other interrupt source

PIR1=0x00;			// Clear all peripheral interrupt flag

T0IE=1;     		// Enable the timer interrupt
PEIE=1;				// Enable all un masked peripheral interrupts
GIE=1;				// Enable Global Interrupts
}


//---------------------------------------------------------------------
//	InitVar() initializes program variables 
//---------------------------------------------------------------------
void InitVar(void)
{
basetime.bit0=0;		// clear basetime bit flag
basetime.bit1=0;
basetime.bit2=0;
basetime.bit3=0;

intcnt0=0;				// clear interrupt counter
intcnt1=0;
intcnt2=0;

num=0;
capture=0;
tmax_cnt=0;
updatelcd(0, FALSE);		// clear display 
}



//---------------------------------------------------------------------
//	Isr() Interrupt service routine
//---------------------------------------------------------------------

void interrupt Isr(void)
{
asm("nop");
asm("nop");
asm("nop");
asm("nop");
TMR0=7;							// 16*usx250=4000us; 255-250+2=7

basetime.bit0=1;				// set base time bit flag
intcnt0++;						// increment interrpt counter

if (intcnt0>9)
	{							// every 10 interrupt					
	intcnt0=0;
	basetime.bit1=1;
	intcnt1++;
	if(intcnt1>4)
		{						// every 50 interrupt
		intcnt1=0;
		basetime.bit2=1;
		intcnt2++;
		if(intcnt2>1)			
			{					// every 100 interrupt
			intcnt2=0;
			basetime.bit3=1;
			}	
		}
	}
T0IF=0;							// clear TMR0 interrupt Flag
}


//---------------------------------------------------------------------
//	Base time 0 routine 4ms
//---------------------------------------------------------------------
void basetime0(void)
{
basetime.bit0=0;
TEST=!TEST;
if(tmax_cnt) tmax_cnt--;
else capture=0;
}

//---------------------------------------------------------------------
//	Base time 1 routine	40ms
//---------------------------------------------------------------------
void basetime1(void)
{
basetime.bit1=0;
}

//---------------------------------------------------------------------
//	Base time 2 routine	240ms
//---------------------------------------------------------------------
void basetime2(void)
{
basetime.bit2=0;
calcrpm();
updatelcd(num, TRUE);
}

//---------------------------------------------------------------------
//	Base time 3 routine	480ms
//---------------------------------------------------------------------
void basetime3(void)
{
basetime.bit3=0;
}

//---------------------------------------------------------------------
// 				Calculate RPM from captured pulse
//---------------------------------------------------------------------
void calcrpm(void)
{
if(capture==0) num=0;			// rpm too low
else
	{
	if (capture<750) num=9999;	// rpm too high
	else num=(unsigned long int) 7500000/capture;
	}
}
